@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2002-2004 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,15 +15,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.59 2003/08/24 18:20:58 chpe Exp $
+ *  $Id: ephy-bookmarks.c,v 1.72 2004/01/28 18:32:07 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ephy-bookmarks.h"
 #include "ephy-file-helpers.h"
+#include "ephy-embed-shell.h"
 #include "ephy-shell.h"
 #include "ephy-history.h"
 #include "ephy-debug.h"
@@ -32,17 +33,21 @@
 #include "ephy-toolbars-model.h"
 #include "ephy-bookmarks-export.h"
 #include "ephy-bookmarks-import.h"
-#include "ephy-autocompletion.h"
-#include "session.h"
+#include "ephy-prefs.h"
+
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#define EPHY_BOOKMARKS_XML_VERSION "1.0"
+#define EPHY_BOOKMARKS_XML_ROOT    "ephy_bookmarks"
+#define EPHY_BOOKMARKS_XML_VERSION "1.01"
 #define BOOKMARKS_SAVE_DELAY (3 * 1000)
 #define MAX_FAVORITES_NUM 10
 
+#define EPHY_BOOKMARKS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARKS, EphyBookmarksPrivate))
+
 struct EphyBookmarksPrivate
 {
 	EphyToolbarsModel *toolbars_model;
@@ -56,15 +61,17 @@ struct EphyBookmarksPrivate
 	EphyNode *keywords;
 	EphyNode *favorites;
 	EphyNode *notcategorized;
+	EphyNode *smartbookmarks;
 	EphyNode *lower_fav;
 	double lower_score;
+
+	guint disable_bookmark_editing_notifier_id;
 };
 
 typedef struct
 {
 	const char *title;
 	const char *location;
-	const char *smart_url;
 } EphyBookmarksBookmarkInfo;
 
 static const EphyBookmarksBookmarkInfo default_bookmarks [] =
@@ -73,7 +80,6 @@ static const EphyBookmarksBookmarkInfo default_bookmarks [] =
 	 * For instance in .nl these should be
 	 * "http://www.google.nl" and "http://www.google.nl/search?q=%s"
 	 */
-
 	{ N_("Search the web"), N_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8") }
 };
 static int n_default_bookmarks = G_N_ELEMENTS (default_bookmarks);
@@ -104,23 +110,18 @@ enum
 
 static guint ephy_bookmarks_signals[LAST_SIGNAL] = { 0 };
 
-static void
-ephy_bookmarks_class_init (EphyBookmarksClass *klass);
-static void
-ephy_bookmarks_init (EphyBookmarks *tab);
-static void
-ephy_bookmarks_finalize (GObject *object);
-static void
-ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface);
+static void ephy_bookmarks_class_init	(EphyBookmarksClass *klass);
+static void ephy_bookmarks_init		(EphyBookmarks *tab);
+static void ephy_bookmarks_finalize	(GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
 GType
 ephy_bookmarks_get_type (void)
 {
-        static GType ephy_bookmarks_type = 0;
+        static GType type = 0;
 
-        if (ephy_bookmarks_type == 0)
+        if (type == 0)
         {
                 static const GTypeInfo our_info =
                 {
@@ -135,76 +136,12 @@ ephy_bookmarks_get_type (void)
                         (GInstanceInitFunc) ephy_bookmarks_init
                 };
 
-		static const GInterfaceInfo autocompletion_source_info =
-		{
-			(GInterfaceInitFunc) ephy_bookmarks_autocompletion_source_init,
-			NULL,
-			NULL
-		};
-
-                ephy_bookmarks_type = g_type_register_static (G_TYPE_OBJECT,
-							      "EphyBookmarks",
-							      &our_info, 0);
-
-		g_type_add_interface_static (ephy_bookmarks_type,
-					     EPHY_TYPE_AUTOCOMPLETION_SOURCE,
-					     &autocompletion_source_info);
+                type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyBookmarks",
+					       &our_info, 0);
         }
 
-        return ephy_bookmarks_type;
-}
-
-static void
-ephy_bookmarks_autocompletion_source_set_basic_key (EphyAutocompletionSource *source,
-						    const gchar *basic_key)
-{
-	/* nothing to do here */
-}
-
-static void
-ephy_bookmarks_autocompletion_source_foreach (EphyAutocompletionSource *source,
-					      const gchar *current_text,
-					      EphyAutocompletionSourceForeachFunc func,
-					      gpointer data)
-{
-	GPtrArray *children;
-	int i;
-	EphyBookmarks *eb = EPHY_BOOKMARKS (source);
-
-	children = ephy_node_get_children (eb->priv->bookmarks);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-		const char *url, *title, *keywords;
-		gboolean smart_url;
-
-		kid = g_ptr_array_index (children, i);
-		url = ephy_node_get_property_string
-			(kid, EPHY_NODE_BMK_PROP_LOCATION);
-		smart_url = ephy_node_get_property_boolean
-			(kid, EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS);
-		title = ephy_node_get_property_string
-			(kid, EPHY_NODE_BMK_PROP_TITLE);
-		keywords = ephy_node_get_property_string
-			(kid, EPHY_NODE_BMK_PROP_KEYWORDS);
-
-		func (source, keywords, title, url, smart_url,
-		      !smart_url, 0, data);
-	}
-	ephy_node_thaw (eb->priv->bookmarks);
-}
-
-static void
-ephy_bookmarks_emit_data_changed (EphyBookmarks *eb)
-{
-	g_signal_emit_by_name (eb, "data-changed");
-}
-
-static void
-ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
-{
-	iface->foreach = ephy_bookmarks_autocompletion_source_foreach;
-	iface->set_basic_key = ephy_bookmarks_autocompletion_source_set_basic_key;
+        return type;
 }
 
 static void
@@ -214,15 +151,15 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 
 	for (i = 0; i < n_default_topics; i++)
 	{
-		ephy_bookmarks_add_keyword (eb, default_topics[i]);
+		ephy_bookmarks_add_keyword (eb, _(default_topics[i]));
 	}
 
 	for (i = 0; i < n_default_bookmarks; i++)
 	{
 		EphyNode *bmk;
 
-		bmk = ephy_bookmarks_add (eb, default_bookmarks[i].title,
-				          default_bookmarks[i].location);
+		bmk = ephy_bookmarks_add (eb, _(default_bookmarks[i].title),
+				          _(default_bookmarks[i].location));
 		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE,
 						  ephy_node_get_id (bmk));
 	}
@@ -304,101 +241,28 @@ ephy_bookmarks_class_init (EphyBookmarksClass *klass)
                                          g_param_spec_object ("toolbars_model",
                                                               "Toolbars model",
                                                               "Toolbars model",
-                                                              EPHY_TOOLBARS_MODEL_TYPE,
+                                                              EPHY_TYPE_TOOLBARS_MODEL,
                                                               G_PARAM_READWRITE));
 
-}
-
-static gboolean
-ephy_bookmarks_load (EphyBookmarks *eb)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-	char *tmp;
-	gboolean result = TRUE;
-
-	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
-		return FALSE;
-
-	doc = xmlParseFile (eb->priv->xml_file);
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	root = xmlDocGetRootElement (doc);
-	child = root->children;
-	
-	tmp = xmlGetProp (root, "version");
-	if (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) != 0)
-	{
-		g_warning ("Old bookmarks database format detected");
-
-		child = NULL;
-		result = FALSE;
-	}
-	g_free (tmp);
-
-	for (; child != NULL; child = child->next)
-	{
-		EphyNode *node;
-
-		node = ephy_node_new_from_xml (eb->priv->db, child);
-	}
-
-	xmlFreeDoc (doc);
-
-	return result;
+	g_type_class_add_private (object_class, sizeof(EphyBookmarksPrivate));
 }
 
 static void
 ephy_bookmarks_save (EphyBookmarks *eb)
 {
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	xmlNodePtr comment;
-	GPtrArray *children;
-	int i;
 
 	LOG ("Saving bookmarks")
 
-	/* save nodes to xml */
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
-
-	root = xmlNewDocNode (doc, NULL, "ephy_bookmarks", NULL);
-	xmlSetProp (root, "version", EPHY_BOOKMARKS_XML_VERSION);
-	xmlDocSetRootElement (doc, root);
-
-	comment = xmlNewDocComment (doc, "Do not rely on this file, it's only for internal use. Use bookmarks.rdf instead.");
-	xmlAddChild (root, comment);
-
-	children = ephy_node_get_children (eb->priv->keywords);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		if (kid != eb->priv->bookmarks &&
-		    kid != eb->priv->favorites &&
-		    kid != eb->priv->notcategorized)
-		{
-			ephy_node_save_to_xml (kid, root);
-		}
-	}
-	ephy_node_thaw (eb->priv->keywords);
-
-	children = ephy_node_get_children (eb->priv->bookmarks);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_save_to_xml (kid, root);
-	}
-	ephy_node_thaw (eb->priv->bookmarks);
-
-	ephy_file_save_xml (eb->priv->xml_file, doc);
-	xmlFreeDoc(doc);
+	ephy_node_db_write_to_xml_safe
+		(eb->priv->db, eb->priv->xml_file,
+		 EPHY_BOOKMARKS_XML_ROOT,
+		 EPHY_BOOKMARKS_XML_VERSION,
+		 "Do not rely on this file, it's only for internal use. Use bookmarks.rdf instead.",
+		 eb->priv->keywords,
+		 3, eb->priv->bookmarks, eb->priv->favorites, eb->priv->notcategorized,
+		 eb->priv->bookmarks,
+		 0,
+		 NULL);
 
 	/* Export bookmarks in rdf */
 	ephy_bookmarks_export_rdf (eb, eb->priv->rdf_file);
@@ -448,12 +312,10 @@ compute_lower_fav (EphyNode *favorites, double *score)
 {
 	GPtrArray *children;
 	int i;
-	EphyEmbedShell *embed_shell;
 	EphyHistory *history;
 	EphyNode *result = NULL;
 
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
-	history = ephy_embed_shell_get_global_history (embed_shell);
+	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
 
 	*score = DBL_MAX;
 	children = ephy_node_get_children (favorites);
@@ -473,7 +335,6 @@ compute_lower_fav (EphyNode *favorites, double *score)
 			result = child;
 		}
 	}
-	ephy_node_thaw (favorites);
 
 	if (result == NULL) *score = 0;
 
@@ -499,7 +360,7 @@ add_to_favorites (EphyBookmarks *eb, EphyNode *node, EphyHistory *eh)
 	url = ephy_node_get_property_string (node, EPHY_NODE_BMK_PROP_LOCATION);
 	score = get_history_item_score (eh, url);
 	full_menu = ephy_node_get_n_children (eb->priv->favorites)
-		    > MAX_FAVORITES_NUM;
+		    >= MAX_FAVORITES_NUM;
 	if (full_menu && score < eb->priv->lower_score) return FALSE;
 
 	if (eb->priv->lower_fav && full_menu)
@@ -514,23 +375,6 @@ add_to_favorites (EphyBookmarks *eb, EphyNode *node, EphyHistory *eh)
 	return TRUE;
 }
 
-static void
-update_favorites_menus ()
-{
-	Session *session;
-	const GList *l;
-
-	session = SESSION (ephy_shell_get_session (ephy_shell));
-	l = session_get_windows (session);
-
-	for (; l != NULL; l = l->next)
-	{
-		EphyWindow *window = EPHY_WINDOW (l->data);
-
-		ephy_window_update_control (window, FavoritesControl);
-	}
-}
-
 static void
 history_site_visited_cb (EphyHistory *gh, const char *url, EphyBookmarks *eb)
 {
@@ -539,23 +383,60 @@ history_site_visited_cb (EphyHistory *gh, const char *url, EphyBookmarks *eb)
 	node = ephy_bookmarks_find_bookmark (eb, url);
 	if (node == NULL) return;
 
-	if (add_to_favorites (eb, node, gh))
+	add_to_favorites (eb, node, gh);
+}
+
+static void
+clear_favorites (EphyBookmarks *bookmarks)
+{
+	EphyNode *node;
+	GPtrArray *children;
+	int i;
+	gboolean was_immutable;
+
+	/* clear the favourites */
+
+	was_immutable = ephy_node_db_is_immutable (bookmarks->priv->db);
+	ephy_node_db_set_immutable (bookmarks->priv->db, FALSE);
+
+	node = bookmarks->priv->favorites;
+	children = ephy_node_get_children (node);
+	for (i = children->len - 1; i >= 0; i--)
 	{
-		update_favorites_menus ();
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+
+		ephy_node_remove_child (node, kid);
 	}
+
+	ephy_node_db_set_immutable (bookmarks->priv->db, was_immutable);
+
+	ephy_bookmarks_update_favorites (bookmarks);
+}
+
+static void
+history_cleared_cb (EphyHistory *history, EphyBookmarks *bookmarks)
+{
+	clear_favorites (bookmarks);
 }
 
 static void
 ephy_setup_history_notifiers (EphyBookmarks *eb)
 {
-	EphyEmbedShell *embed_shell;
 	EphyHistory *history;
 
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
-	history = ephy_embed_shell_get_global_history (embed_shell);
+	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
+
+	if (ephy_history_is_enabled (history) == FALSE)
+	{
+		clear_favorites (eb);
+	}
 
 	g_signal_connect (history, "visited",
 			  G_CALLBACK (history_site_visited_cb), eb);
+	g_signal_connect (history, "cleared",
+			  G_CALLBACK (history_cleared_cb), eb);
 }
 
 static void
@@ -563,7 +444,6 @@ bookmarks_changed_cb (EphyNode *node,
 		      EphyNode *child,
 		      EphyBookmarks *eb)
 {
-	ephy_bookmarks_emit_data_changed (eb);
 	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
 
@@ -573,7 +453,6 @@ bookmarks_removed_cb (EphyNode *node,
 		      guint old_index,
 		      EphyBookmarks *eb)
 {
-	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
@@ -609,7 +488,6 @@ get_topics_list (EphyBookmarks *eb,
 			*no_topics = FALSE;
 		}
 	}
-	ephy_node_thaw (eb->priv->keywords);
 
 	return g_string_free (list, FALSE);
 }
@@ -657,20 +535,34 @@ topics_removed_cb (EphyNode *node,
 
 		g_free (list);
 	}
-	ephy_node_thaw (child);
 
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
+static void
+update_bookmark_editing (EphyBookmarks *eb)
+{
+	g_object_set (G_OBJECT (eb->priv->db),
+		      "immutable", eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING),
+		      NULL);
+}
+
+static void
+disable_bookmark_editing_notifier (GConfClient *client,
+				   guint cnxn_id,
+				   GConfEntry *entry,
+				   EphyBookmarks *eb)
+{
+	update_bookmark_editing (eb);
+}
+
 static void
 ephy_bookmarks_init (EphyBookmarks *eb)
 {
 	GValue value = { 0, };
 	EphyNodeDb *db;
 
-	LOG ("Init");
-
-        eb->priv = g_new0 (EphyBookmarksPrivate, 1);
+	eb->priv = EPHY_BOOKMARKS_GET_PRIVATE (eb);
 	eb->priv->toolbars_model = NULL;
 
 	db = ephy_node_db_new (EPHY_NODE_DB_BOOKMARKS);
@@ -753,15 +645,24 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	g_value_unset (&value);
 	ephy_node_add_child (eb->priv->keywords, eb->priv->notcategorized);
 
-	if (!ephy_bookmarks_load (eb))
+	/* Smart bookmarks */
+	eb->priv->smartbookmarks = ephy_node_new_with_id (db, SMARTBOOKMARKS_NODE_ID);
+	ephy_node_ref (eb->priv->smartbookmarks);
+
+	if (ephy_node_db_load_from_file (eb->priv->db, eb->priv->xml_file,
+					 EPHY_BOOKMARKS_XML_ROOT,
+					 EPHY_BOOKMARKS_XML_VERSION) == FALSE)
 	{
-		if (!ephy_bookmarks_import_rdf (eb, eb->priv->rdf_file))
+		if (ephy_bookmarks_import_rdf (eb, eb->priv->rdf_file) == FALSE)
 		{
-			eb->priv->init_defaults = !ephy_bookmarks_load (eb);
+			eb->priv->init_defaults = TRUE;
 		}
 	}
 
-	ephy_bookmarks_emit_data_changed (eb);
+	eb->priv->disable_bookmark_editing_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING,
+		 (GConfClientNotifyFunc)disable_bookmark_editing_notifier, eb);
+	update_bookmark_editing (eb);
 
 	ephy_setup_history_notifiers (eb);
 	ephy_bookmarks_update_favorites (eb);
@@ -770,13 +671,9 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 static void
 ephy_bookmarks_finalize (GObject *object)
 {
-        EphyBookmarks *eb;
+	EphyBookmarks *eb = EPHY_BOOKMARKS (object);
 
-	g_return_if_fail (IS_EPHY_BOOKMARKS (object));
-
-	eb = EPHY_BOOKMARKS (object);
-
-        g_return_if_fail (eb->priv != NULL);
+	eel_gconf_notification_remove (eb->priv->disable_bookmark_editing_notifier_id);
 
 	if (eb->priv->save_timeout_id != 0)
 	{
@@ -801,8 +698,6 @@ ephy_bookmarks_finalize (GObject *object)
 	g_free (eb->priv->xml_file);
 	g_free (eb->priv->rdf_file);
 
-        g_free (eb->priv);
-
 	LOG ("Bookmarks finalized")
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -811,29 +706,40 @@ ephy_bookmarks_finalize (GObject *object)
 EphyBookmarks *
 ephy_bookmarks_new ()
 {
-	EphyBookmarks *tab;
+	EphyBookmarks *eb;
 
-	tab = EPHY_BOOKMARKS (g_object_new (EPHY_BOOKMARKS_TYPE, NULL));
+	eb = EPHY_BOOKMARKS (g_object_new (EPHY_TYPE_BOOKMARKS, NULL));
 
-	return tab;
+	return eb;
 }
 
 static void
-update_has_smart_address (EphyNode *bmk, const char *address)
+update_has_smart_address (EphyBookmarks *bookmarks, EphyNode *bmk, const char *address)
 {
+	EphyNode *smart_bmks;
 	gboolean smart = FALSE;
-	GValue value = { 0, };
+
+	smart_bmks = bookmarks->priv->smartbookmarks;
 
 	if (address)
 	{
 		smart = strstr (address, "%s") != NULL;
 	}
 
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, smart);
-	ephy_node_set_property (bmk, EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS,
-			        &value);
-	g_value_unset (&value);
+	if (smart)
+	{
+		if (!ephy_node_has_child (smart_bmks, bmk))
+		{
+			ephy_node_add_child (smart_bmks, bmk);
+		}
+	}
+	else
+	{
+		if (ephy_node_has_child (smart_bmks, bmk))
+		{
+			ephy_node_add_child (smart_bmks, bmk);
+		}
+	}
 }
 
 EphyNode *
@@ -846,6 +752,8 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 
 	bm = ephy_node_new (eb->priv->db);
 
+	if (bm == NULL) return NULL;
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, title);
 	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_TITLE,
@@ -858,12 +766,11 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 			        &value);
 	g_value_unset (&value);
 
-	update_has_smart_address (bm, url);
+	update_has_smart_address (eb, bm, url);
 
 	ephy_node_add_child (eb->priv->bookmarks, bm);
 	ephy_node_add_child (eb->priv->notcategorized, bm);
 
-	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 	ephy_bookmarks_save_delayed (eb, 0);
 
@@ -883,7 +790,7 @@ ephy_bookmarks_set_address (EphyBookmarks *eb,
 			        &value);
 	g_value_unset (&value);
 
-	update_has_smart_address (bookmark, address);
+	update_has_smart_address (eb, bookmark, address);
 }
 
 EphyNode *
@@ -893,7 +800,7 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 	GPtrArray *children;
 	int i;
 
-	g_return_val_if_fail (IS_EPHY_BOOKMARKS (eb), NULL);
+	g_return_val_if_fail (EPHY_IS_BOOKMARKS (eb), NULL);
 	g_return_val_if_fail (eb->priv->bookmarks != NULL, NULL);
 	g_return_val_if_fail (url != NULL, NULL);
 
@@ -909,11 +816,9 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 
 		if (location != NULL && strcmp (url, location) == 0)
 		{
-			ephy_node_thaw (eb->priv->bookmarks);
 			return kid;
 		}
 	}
-	ephy_node_thaw (eb->priv->bookmarks);
 
 	return NULL;
 }
@@ -989,6 +894,17 @@ options_find_next_option (const char *current)
 	return (char *) ret;
 }
 
+static char *
+options_find_first_option (const char *input)
+{
+	const char *start;
+
+	if (!input) return NULL;
+	start = strrchr (input, '{');
+	if (!start || !(*start)) return NULL;
+	return options_skip_spaces (start+1);
+}
+
 /**
  * Very simple parser for option strings in the
  * form a=b,c=d,e="f g",...
@@ -998,7 +914,7 @@ smart_url_options_get (const gchar *options, const gchar *option)
 {
 	gchar *ret = NULL;
 	gsize optionlen = strlen (option);
-	const gchar *current = options_skip_spaces (options);
+	const gchar *current = options_find_first_option (options_skip_spaces (options));
 
 	while (current)
 	{
@@ -1074,6 +990,7 @@ ephy_bookmarks_solve_smart_url (EphyBookmarks *eb,
 
 	arg = g_convert (content, strlen (content),
 			 encoding, "UTF-8", NULL, NULL, NULL);
+	if (arg == NULL) return NULL;
 	escaped_arg = gnome_vfs_escape_string (arg);
 
 	t1 = smarturl_only;
@@ -1102,6 +1019,8 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 
 	key = ephy_node_new (eb->priv->db);
 
+	if (key == NULL) return NULL;
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, name);
 	ephy_node_set_property (key, EPHY_NODE_KEYWORD_PROP_NAME,
@@ -1210,7 +1129,6 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 			 node = kid;
 		 }
 	}
-	ephy_node_thaw (eb->priv->keywords);
 
 	return node;
 }
@@ -1278,6 +1196,12 @@ ephy_bookmarks_unset_keyword (EphyBookmarks *eb,
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
+EphyNode *
+ephy_bookmarks_get_smart_bookmarks (EphyBookmarks *eb)
+{
+	return eb->priv->smartbookmarks;
+}
+
 EphyNode *
 ephy_bookmarks_get_keywords (EphyBookmarks *eb)
 {
